@@ -29,9 +29,17 @@ import dbus
 import xapian
 import gobject
 
+try:
+    import cjson
+    has_cjson = True
+except ImportError:
+    import simplejson
+    has_cjson = False
+
 from olpc.datastore.xapianindex import IndexManager
 from olpc.datastore import bin_copy
 from olpc.datastore import utils
+from olpc.datastore import model
 
 # changing this pattern impacts _targetFile
 filename_attempt_pattern = re.compile('\(\d+\)$')
@@ -214,7 +222,11 @@ class FileBackingStore(BackingStore):
         instead of a method parameter because this is less invasive for Update 1.
         """
         self.current_user_id = None
-        
+
+        # source for an idle callback that exports to the file system the
+        # metadata from the index
+        self._export_metadata_source = None
+
     # Informational
     def descriptor(self):
         """return a dict with atleast the following keys
@@ -269,11 +281,13 @@ class FileBackingStore(BackingStore):
         if 'uri' not in desc: desc['uri'] = self.uri
         if 'title' not in desc: desc['title'] = self.uri
 
+        # TODO: Would be better to check if the device is present and
+        # don't try to update the descriptor file if it's not.
         try:
             fp = open(fn, 'w')
             pickle.dump(desc, fp)
             fp.close()
-        except Exception:
+        except IOError, e:
             logging.error('Unable to write descriptor:\n' + \
                 ''.join(traceback.format_exception(*sys.exc_info())))
 
@@ -324,7 +338,28 @@ class FileBackingStore(BackingStore):
             im.connect(index_name)
 
             self.indexmanager = im
-            
+
+        # Check that all entries have their metadata in the file system.
+        if not os.path.exists(os.path.join(self.base, '.metadata.exported')):
+            uids_to_export = []
+            uids = self.indexmanager.get_all_ids()
+
+            for uid in uids:
+                if not os.path.exists(os.path.join(self.base, uid + '.metadata')):
+                    uids_to_export.append(uid)
+
+            if uids_to_export:
+                self._export_metadata_source = gobject.idle_add(
+                        self._export_metadata, uids_to_export)
+            else:
+                open(os.path.join(self.base, '.metadata.exported'), 'w').close()
+
+    def _export_metadata(self, uids_to_export):
+        uid = uids_to_export.pop()
+        props = self.indexmanager.get(uid).properties
+        self._store_metadata(uid, props)
+        return len(uids_to_export) > 0
+
     def bind_to(self, datastore):
         ## signal from datastore that we are being bound to it
         self.datastore = datastore
@@ -374,7 +409,7 @@ class FileBackingStore(BackingStore):
         if use_instance_dir:
             if not self.current_user_id:
                 raise ValueError("Couldn't determine the current user uid.")
-            base = os.path.join('/activities', 'uid_to_instance_dir',
+            base = os.path.join(os.environ['HOME'], 'isolation', '1', 'uid_to_instance_dir',
                                 str(self.current_user_id))
         else:
             profile = os.environ.get('SUGAR_PROFILE', 'default')
@@ -408,14 +443,13 @@ class FileBackingStore(BackingStore):
             if e.errno != errno.EPERM:
                 raise
 
-        # Try to link from the original file to the targetpath. This can fail if
-        # the file is in a different filesystem. Do a copy instead.
+        # Try to hard link from the original file to the targetpath. This can
+        # fail if the file is in a different filesystem. Do a symlink instead.
         try:
             os.link(path, targetpath)
         except OSError, e:
             if e.errno == errno.EXDEV:
-                shutil.copy(path, targetpath)
-                os.chmod(targetpath, 0604)
+                os.symlink(path, targetpath)
             else:
                 raise
             
@@ -498,8 +532,33 @@ class FileBackingStore(BackingStore):
             c.update(line)
         fp.close()
         return c.hexdigest()
-        
+
     # File Management API
+    def _encode_json(self, metadata, file_path):
+        if has_cjson:
+            f = open(file_path, 'w')
+            f.write(cjson.encode(metadata))
+            f.close()
+        else:
+            simplejson.dump(metadata, open(file_path, 'w'))
+
+    def _store_metadata(self, uid, props):
+        t = time.time()
+        temp_path = os.path.join(self.base, '.temp_metadata')
+        props = props.copy()
+        for property_name in model.defaultModel.get_external_properties():
+            if property_name in props:
+                del props[property_name]
+        self._encode_json(props, temp_path)
+        path = os.path.join(self.base, uid + '.metadata')
+        os.rename(temp_path, path)
+        logging.debug('exported metadata: %r s.' % (time.time() - t))
+
+    def _delete_metadata(self, uid):
+        path = os.path.join(self.base, uid + '.metadata')
+        if os.path.exists(path):
+            os.unlink(path)
+
     def _create_completion(self, uid, props, completion, exc=None, path=None):
         if exc:
             completion(exc)
@@ -515,6 +574,7 @@ class FileBackingStore(BackingStore):
         if completion is None:
             raise RuntimeError("Completion must be valid for async create")
         uid = self.indexmanager.index(props)
+        self._store_metadata(uid, props)
         props['uid'] = uid
         if filelike:
             if isinstance(filelike, basestring):
@@ -529,6 +589,7 @@ class FileBackingStore(BackingStore):
     def create(self, props, filelike, can_move=False):
         if filelike:
             uid = self.indexmanager.index(props)
+            self._store_metadata(uid, props)
             props['uid'] = uid
             if isinstance(filelike, basestring):
                 # lets treat it as a filename
@@ -538,7 +599,9 @@ class FileBackingStore(BackingStore):
             self.indexmanager.index(props, path)
             return uid
         else:
-            return self.indexmanager.index(props)
+            uid = self.indexmanager.index(props)
+            self._store_metadata(uid, props)
+            return uid
     
     def get(self, uid, env=None, allowMissing=False, includeFile=False):
         content = self.indexmanager.get(uid)
@@ -573,6 +636,7 @@ class FileBackingStore(BackingStore):
             raise RuntimeError("Completion must be valid for async update")
 
         props['uid'] = uid
+        self._store_metadata(uid, props)
         if filelike:
             uid = self.indexmanager.index(props, filelike)
             props['uid'] = uid
@@ -588,6 +652,7 @@ class FileBackingStore(BackingStore):
 
     def update(self, uid, props, filelike=None, can_move=False):
         props['uid'] = uid
+        self._store_metadata(uid, props)
         if filelike:
             if isinstance(filelike, basestring):
                 # lets treat it as a filename
@@ -598,14 +663,25 @@ class FileBackingStore(BackingStore):
         else:
             self.indexmanager.index(props)
 
+    def _delete_external_properties(self, uid):
+        external_properties = model.defaultModel.get_external_properties()
+        for property_name in external_properties:
+            file_path = os.path.join(self.base, property_name, uid)
+            if os.path.exists(file_path):
+                logging.debug('deleting external property: %r' % file_path)
+                os.unlink(file_path)
+
     def delete(self, uid, allowMissing=True):
+        self._delete_external_properties(uid)
+        self._delete_metadata(uid)
+
         self.indexmanager.delete(uid)
         path = self._translatePath(uid)
         if os.path.exists(path):
             os.unlink(path)
         else:
             if not allowMissing:
-                raise KeyError("object for uid:%s missing" % uid)            
+                raise KeyError("object for uid:%s missing" % uid)
         
     def get_uniquevaluesfor(self, propertyname):
         return self.indexmanager.get_uniquevaluesfor(propertyname)
@@ -639,6 +715,8 @@ class FileBackingStore(BackingStore):
         return self.indexmanager.get_all_ids()
     
     def stop(self):
+        if self._export_metadata_source is not None:
+            gobject.source_remove(self._export_metadata_source)
         self.indexmanager.stop()
 
     def complete_indexing(self):
@@ -887,6 +965,8 @@ class InplaceFileBackingStore(FileBackingStore):
         self.indexmanager.index(props, path)
         
     def delete(self, uid):
+        self._delete_external_properties(uid)
+
         c = self.indexmanager.get(uid)
         path = c.get_property('filename', None)
         self.indexmanager.delete(uid)
