@@ -125,7 +125,15 @@ class IndexManager(object):
         logging.debug('IndexManager.stop()')
         self.flush(force=True)
         self.stopIndexer(force)
-        self.write_index.close()
+
+        # TODO: Would be better to check if the device is present and
+        # don't try to close if it's not.
+        try:
+            self.write_index.close()
+        except _xapian.DatabaseError, e:
+            logging.debug('Index close failed:\n' + \
+                ''.join(traceback.format_exception(*sys.exc_info())))
+
         #self.read_index.close()
         # XXX: work around for xapian not having close() this will
         # change in the future in the meantime we delete the
@@ -166,11 +174,15 @@ class IndexManager(object):
         self.deltact += 1
         if force or self.deltact > FLUSH_THRESHOLD:
             with self._write_lock:
+
+                # TODO: Would be better to check if the device is present and
+                # don't try to flush if it's not.
                 try:
                    self.write_index.flush()
-                except Exception:
-                    logging.error('Index flush failed:\n' + \
+                except _xapian.DatabaseError, e:
+                    logging.debug('Index flush failed:\n' + \
                         ''.join(traceback.format_exception(*sys.exc_info())))
+
                 #self.read_index.reopen()
                 self.deltact = 0
         else:
@@ -455,7 +467,8 @@ class IndexManager(object):
             if order_by: order_by = order_by[0]
             else: order_by = None
                 
-        results = ri.search(q, start_index, end_index, sortby=order_by)
+        results = ri.search(q, start_index, end_index, sortby=order_by,
+                            checkatleast=sys.maxint)
         count = results.matches_estimated
 
         # map the result set to model.Content items
